@@ -61,6 +61,7 @@
 
 sfun _pure_syscall=syscall;
 sfun _pure_native_syscall=syscall;
+extern sfun _pure_socketcall;
 
 static int _pageshift()
 {
@@ -94,7 +95,7 @@ long _pure_debug_printf(const char *format, ...)
 // open must consider two mode of calling: with two or three arguments
 int open(const char* pathname,int flags,...){
 	va_list arg_list;
-	if( flags |  O_CREAT ){
+	if( flags &  O_CREAT ){
 		mode_t mode;
 		va_start(arg_list,flags);
 		mode = va_arg(arg_list,mode_t);
@@ -107,7 +108,7 @@ int open(const char* pathname,int flags,...){
 
 int open64(const char* pathname,int flags,...){
 	va_list arg_list;
-	if( flags |  O_CREAT ){
+	if( flags &  O_CREAT ){
 		mode_t mode;
 		va_start(arg_list,flags);
 		mode = va_arg(arg_list,mode_t);
@@ -160,6 +161,12 @@ int dup2(int oldfd, int newfd){
 	return _pure_syscall(__NR_dup2, oldfd, newfd);
 }
 
+#ifdef __NR_dup3
+int dup3(int oldfd, int newfd, int flags){
+	  return _pure_syscall(__NR_dup3, oldfd, newfd,flags);
+}
+#endif
+
 /* When <sys/stat.h> is included, inline #defines for {,l,f}stat{,64} are
  * inserted and they make calls to __{,l,f}xstat{,64}. So we don't need
  * to define them.
@@ -278,6 +285,7 @@ int __lxstat64(int ver,const char* pathname,struct stat64* buf){
 int __fxstat64 (int ver, int fildes, struct stat64 *buf){
 	return _pure_syscall(MAKE_NAME(__NR_f, arch_stat64), fildes, buf);
 }
+/* end of unreadable code */
 
 int mknod(const char *pathname, mode_t mode, dev_t dev) {
 	return _pure_syscall(__NR_mknod,pathname,mode,dev);
@@ -290,6 +298,21 @@ int __xmknod (int ver, const char *path, mode_t mode, dev_t *dev) {
 int access(const char* pathname,int mode){
 	return _pure_syscall(__NR_access,pathname,mode);
 }
+int __access(const char* pathname,int mode){
+	return access(pathname,mode);
+}
+
+#ifdef __NR_faccessat
+int euidaccess(const char *pathname, int mode){
+	return _pure_syscall(__NR_faccessat,AT_FDCWD,pathname,mode,AT_EACCESS);
+}
+int eaccess(const char *pathname, int mode){
+	return euidaccess(pathname,mode);
+}
+int __euidaccess(const char *pathname, int mode){
+	return euidaccess(pathname,mode);
+}
+#endif
 
 ssize_t readlink(const char* pathname,char* buf, size_t bufsize){
 	return _pure_syscall(__NR_readlink,pathname,buf,bufsize);
@@ -393,6 +416,81 @@ ssize_t pwrite(int fs,const void* buf, size_t count, __off_t offset){
 }
 #endif
 
+#ifdef __NR_preadv
+ssize_t preadv64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
+	ssize_t rv=_pure_syscall(__NR_preadv,fs,iov,iovcnt,
+#ifdef __NR_pread64
+#if defined(__powerpc__)
+			0,
+#endif
+			__LONG_LONG_PAIR( (__off_t)(offset>>32),(__off_t)(offset&0xffffffff))
+#else
+			offset
+#endif
+			);
+	if (rv==-1 && errno==ENOSYS) {
+		ssize_t totalsize;
+		unsigned char *buf,*scan;
+		int i;
+		for (i=totalsize=0; i<iovcnt; i++)
+			totalsize+=iov[i].iov_len;
+		buf=malloc(totalsize);
+		if (buf==NULL) {
+			errno=ENOMEM;
+			return -1;
+		}
+		totalsize=pread64(fs, buf, totalsize, offset);
+		rv=totalsize;
+		for (i=0,scan=buf; i<iovcnt && totalsize>0; i++,
+				scan+=iov[i].iov_len,
+				totalsize-=iov[i].iov_len) 
+			memcpy(iov[i].iov_base, scan, iov[i].iov_len);
+		free(buf);
+	}
+	return rv;
+}
+
+ssize_t preadv(int fs,const struct iovec *iov, int iovcnt, __off_t offset){
+	return preadv64(fs,iov,iovcnt,(__off64_t)offset);
+}
+#endif
+
+#ifdef __NR_pwritev
+ssize_t pwritev64(int fs,const struct iovec *iov, int iovcnt, __off64_t offset){
+	ssize_t rv=_pure_syscall(__NR_pwritev,fs,iov,iovcnt,
+#ifdef __NR_pwrite64
+#if defined(__powerpc__)
+			0,    
+#endif
+			__LONG_LONG_PAIR( (__off_t)(offset>>32),(__off_t)(offset&0xffffffff))
+#else
+			offset
+#endif
+			);
+	if (rv==-1 && errno==ENOSYS) {
+		ssize_t totalsize;
+		unsigned char *buf,*scan;
+		int i;
+		for (i=totalsize=0; i<iovcnt; i++)
+			totalsize+=iov[i].iov_len;
+		buf=malloc(totalsize);
+		if (buf==NULL) {
+			errno=ENOMEM;
+			return -1;
+		}
+		for (i=0,scan=buf; i<iovcnt; i++, scan+=iov[i].iov_len)
+			memcpy(scan, iov[i].iov_base, iov[i].iov_len);
+		rv=pwrite64(fs, buf, totalsize, offset);
+		free(buf);
+	}
+	return rv;
+}
+
+ssize_t pwritev(int fs,const struct iovec *iov, int iovcnt, __off_t offset){
+	return pwritev64(fs,iov,iovcnt,(__off64_t)offset);
+}
+#endif
+
 int getdents(int fd, long dirp,unsigned int count){
 	return _pure_syscall(__NR_getdents,fd,dirp,count);
 }
@@ -513,7 +611,19 @@ int fcntl(int fd, int cmd, ...){
 	arg1=va_arg(ap,  long int);
 	arg2=va_arg(ap,  long int);
 	va_end(ap);
+	/* XXX Check the fcntl->fcntl64 conversion */
+#ifdef __NR_fcntl64
+	switch (cmd) {
+		case F_GETLK:
+		case F_SETLK:
+		case F_SETLKW:
+			return _pure_syscall(__NR_fcntl,fd,cmd,arg1,arg2);
+		default:
+			return _pure_syscall(__NR_fcntl64,fd,cmd,arg1,arg2);
+	}
+#else
 	return _pure_syscall(__NR_fcntl,fd,cmd,arg1,arg2);
+#endif
 }
 
 #ifdef __NR_fcntl64
@@ -588,25 +698,37 @@ int setegid(gid_t egid){
 uid_t getuid(void) {
 	return _pure_syscall(__NR_getuid);
 }
+uid_t __getuid(void) {
+	return getuid();
+}
 
 gid_t getgid(void) {
 	return _pure_syscall(__NR_getgid);
 }
+gid_t __getgid(void) {
+	return getgid();
+}
 
 uid_t geteuid(void) {
 	return _pure_syscall(__NR_geteuid);
 }
+uid_t __geteuid(void) {
+	return geteuid();
+}
 
 gid_t getegid(void) {
 	return _pure_syscall(__NR_getegid);
 }
+gid_t __getegid(void) {
+	return getegid();
+}
 
 int setreuid(uid_t ruid, uid_t euid){
 	return _pure_syscall(__NR_setreuid,ruid,euid);
 }
 
 int setregid(gid_t rgid, gid_t egid){
-	return _pure_syscall(__NR_setresgid,rgid,egid);
+	return _pure_syscall(__NR_setregid,rgid,egid);
 }
 
 int setresuid(uid_t ruid, uid_t euid, uid_t suid){
@@ -621,6 +743,12 @@ int pipe(int filedes[2]) {
 	return _pure_syscall(__NR_pipe,filedes);
 }
 
+#ifdef __NR_pipe2
+int pipe2(int filedes[2],int flags) {
+	  return _pure_syscall(__NR_pipe2,filedes,flags);
+}
+#endif
+
 mode_t umask(mode_t mask){
 	return _pure_syscall(__NR_umask,mask);
 }
@@ -651,7 +779,11 @@ pid_t fork(void){
 }
 
 pid_t vfork(void){
-	return _pure_syscall(__NR_vfork);
+	 /*The  BSD  man  page  states:  "This system call will be eliminated when
+	 * proper system sharing mechanisms are  implemented.   Users  should  not
+	 * depend  on  the memory sharing semantics of vfork() as it will, in that
+	 * case, be made synonymous to fork(2)." */
+	return _pure_syscall(__NR_fork);
 }
 
 time_t time(time_t *t){
@@ -786,7 +918,7 @@ int getpriority(__priority_which_t which, id_t who){
 }
 
 int setpriority(__priority_which_t which, id_t who, int prio){
-	return _pure_syscall(__NR_getpriority,which,who,prio);
+	return _pure_syscall(__NR_setpriority,which,who,prio);
 }
 
 int statfs(const char *path, struct statfs *buf){
@@ -1082,6 +1214,146 @@ int ftime(struct timeb *tp){
 	return rv;
 }
 
+/* *at syscalls */
+
+#ifdef __NR_openat
+int openat(int dirfd,const char* pathname,int flags,...){
+	va_list arg_list;
+	if( flags &  O_CREAT ){
+		mode_t mode;
+		va_start(arg_list,flags);
+		mode = va_arg(arg_list,mode_t);
+		va_end(arg_list);
+		return _pure_syscall(__NR_openat,dirfd,pathname,flags,mode);
+	}
+	else
+		return _pure_syscall(__NR_openat,dirfd,pathname,flags);
+}
+
+int openat64(int dirfd,const char* pathname,int flags,...){
+	va_list arg_list;
+	if( flags &  O_CREAT ){
+		mode_t mode;
+		va_start(arg_list,flags);
+		mode = va_arg(arg_list,mode_t);
+		va_end(arg_list);
+		return _pure_syscall(__NR_openat,dirfd,pathname,flags|O_LARGEFILE,mode);
+	}
+	else
+		return _pure_syscall(__NR_openat,dirfd,pathname,flags|O_LARGEFILE);
+}
+#endif
+
+#ifdef __NR_mkdirat
+int mkdirat(int dirfd,const char* pathname,mode_t mode){
+	  return _pure_syscall(__NR_mkdirat,dirfd,pathname,mode);
+}
+#endif
+
+#ifdef __NR_mknodat
+int mknodat(int dirfd,const char *pathname, mode_t mode, dev_t dev) {
+	  return _pure_syscall(__NR_mknodat,dirfd,pathname,mode,dev);
+}
+#endif
+
+#ifdef __NR_fchownat
+int fchownat(int dirfd, const char *pathname, uid_t owner, gid_t group, int flags) {
+	return _pure_syscall(__NR_fchownat,dirfd,pathname,owner,group,flags);
+}
+#endif
+
+#ifdef __NR_futimesat
+int futimesat(int dirfd, const char *pathname, const struct timeval times[2]) {
+	return _pure_syscall(__NR_futimesat,dirfd,pathname,times);
+}
+#endif
+
+#ifdef __NR_fstatat64
+#define __NR_FSTATAT64 __NR_fstatat64
+#endif
+#ifdef __NR_newfstatat
+#define __NR_FSTATAT64 __NR_newfstatat
+#endif
+#ifdef __NR_FSTATAT64
+int fstatat(int dirfd, const char *pathname, struct stat *buf, int flags) {
+	return _pure_syscall(__NR_FSTATAT64,dirfd,pathname,buf,flags);
+}
+int __fxstatat64 (int ver, int dirfd, const char *pathname, struct stat64 *buf, int flags){
+	return _pure_syscall(__NR_FSTATAT64,dirfd,pathname,buf,flags);
+}
+int __fxstatat(int ver, int fildes, const char *pathname, struct stat* buf_stat,int flags)
+{
+	IFNOT64(struct stat64 *buf_stat64 = alloca(sizeof(struct stat64));)
+	int rv;
+	switch(ver)
+	{
+		case _STAT_VER_LINUX:
+			rv = _pure_syscall(__NR_FSTATAT64, fildes, pathname, MAKE_NAME(buf_, arch_stat64), flags);
+			break;
+
+		default:
+			fprintf(stderr, "*** BUG! *** __fxstatat can't manage version %d!\n", ver);
+			abort();
+	}
+	if (rv >= 0)
+		arch_stat64_2_stat(MAKE_NAME(buf_, arch_stat64), buf_stat);
+	return rv;
+}
+#endif
+
+#ifdef __NR_unlinkat
+int unlinkat(int dirfd, const char *pathname, int flags){
+	return _pure_syscall(__NR_unlinkat,dirfd,pathname,flags);
+}
+#endif
+
+#ifdef __NR_renameat
+int renameat(int olddirfd, const char *oldpath,int newdirfd, const char *newpath){
+	return _pure_syscall(__NR_renameat,olddirfd,oldpath,newdirfd,newpath);
+}
+#endif
+
+#ifdef __NR_linkat
+int linkat(int olddirfd, const char *oldpath,int newdirfd, const char *newpath, int flags){
+	return _pure_syscall(__NR_linkat,oldpath,newdirfd,newpath,flags);
+}
+#endif
+
+#ifdef __NR_symlinkat
+int symlinkat(const char *oldpath, int newdirfd, const char *newpath){
+	return _pure_syscall(__NR_symlinkat,oldpath,newdirfd,newpath);
+}
+#endif
+
+#ifdef __NR_readlinkat
+ssize_t readlinkat(int dirfd, const char *pathname,char *buf, size_t bufsiz){
+	return _pure_syscall(__NR_readlinkat,dirfd,pathname,bufsiz);
+}
+#endif
+
+#ifdef __NR_fchmodat
+int fchmodat(int dirfd, const char *pathname, mode_t mode, int flags){
+	return _pure_syscall(__NR_fchmodat,dirfd,pathname,mode,flags);
+}
+#endif
+
+#ifdef __NR_faccessat
+int faccessat(int dirfd, const char *pathname, int mode, int flags){
+	return _pure_syscall(__NR_faccessat,dirfd,pathname,mode,flags);
+}
+#endif
+
+#ifdef __NR_utimensat
+int utimensat(int dirfd, const char *pathname, const struct timespec times[2], int flags){
+	return _pure_syscall(__NR_utimensat,dirfd,pathname,times,flags);
+}
+#endif
+
+int raise(int sig) {
+	return kill(getpid(), sig);
+}
+
+static void init(void);
 long int syscall(long int n,...)
 {
 	long int arg0,arg1,arg2,arg3,arg4,arg5;
@@ -1095,9 +1367,55 @@ long int syscall(long int n,...)
 	arg5=va_arg(ap, long int);
 	va_end(ap);
 	if (__builtin_expect(_pure_native_syscall == syscall,0))
-		_pure_native_syscall=dlsym(RTLD_NEXT,"syscall");
+		init();
 	if (__builtin_expect(_pure_syscall == syscall,0))
 		return _pure_native_syscall(n,arg0,arg1,arg2,arg3,arg4,arg5);
 	else
 		return _pure_syscall(n,arg0,arg1,arg2,arg3,arg4,arg5);
 }
+
+sfun _pure_start(sfun pure_syscall,sfun pure_socketcall,int flags)
+{
+	int fdtmp;
+	if (__builtin_expect(_pure_native_syscall == syscall,0))
+		init();
+	if (flags & PUREFLAG_STDIN) {
+		fdtmp=dup(fileno(stdin));
+		dup2(fdtmp,STDIN_FILENO);
+		stdin=fdopen(STDIN_FILENO,"r");
+		if (isatty(STDIN_FILENO))
+			setlinebuf(stdin);
+		close(fdtmp);
+	}
+
+	if (flags & PUREFLAG_STDOUT) {
+		fdtmp=dup(fileno(stdout));
+		dup2(fdtmp,STDOUT_FILENO);
+		stdout=fdopen(STDOUT_FILENO,"w");
+		if (isatty(STDOUT_FILENO))
+			setlinebuf(stdout);
+		close(fdtmp);
+	}
+
+	if (flags & PUREFLAG_STDERR) {
+		fdtmp=dup(fileno(stderr));
+		dup2(fdtmp,STDERR_FILENO);
+		stderr=fdopen(STDERR_FILENO,"a");
+		if (isatty(STDERR_FILENO))
+			setlinebuf(stderr);
+		close(fdtmp);
+	}
+	_pure_syscall=pure_syscall;
+	if (pure_socketcall)
+		_pure_socketcall=pure_socketcall;
+	return _pure_native_syscall;
+}
+
+	static void
+	__attribute__ ((constructor))
+init (void)
+{
+	if (__builtin_expect(_pure_native_syscall == syscall,0))
+		_pure_native_syscall=dlsym(RTLD_NEXT,"syscall");
+}
+
